#!/bin/bash
#####################################################################
# build_gc3_linux.sh - Build GnuCobol 3.x in Linux Mint 19.2/3
# ------------------------------------------------------------------
# This script should work for current and future versions of GnuCobol,
# with only minor modification to the parameters defined below, and 
# possible modifications to the "apt install" list.
#
# Syntax: build_gc3_linux.sh [-cfolder_for_unpacking_tarballs]
#                            [-tfolder_for_downloading_tarballs]
#                            [-lfolder_for_installing_libraries]
#                            [BDB | VBISAM | NODB] [GMP | MPIR]
#                            [WIDE] [cJSON | cJSON=no] [XML2 | XML2=no] 
#                            [GC30 | GC31 | GC32]
#
# Defaults:
#      cobfolder=/home/$USER/cob   #folder where tarballs will be unpacked 
#      tarfolder=/home/$USER/cob"  #folder where tarballs will be downloaded
#      libfolder=/usr/local        #folder where libraries will be installed
#
#      specifying different libfolders allows different installations to co-exist 
#       
#      BDB MPIR cJSON XML2 GC31
#
# All command-line parameters are optional. Folder names are case sensitive.
# The parameters may be entered in any order. 
#
# Command-line parameters override the script variables defined below. In
# general, command-line parameters are used to override variables that are
# likely to change for each user, and for users who do multiple builds. 
# Variables that are more static (such as program versions and download
# urls) must be changed in the script.
# ------------------------------------------------------------------
# Author: Carl D. Goldin <cdgoldin@gmail.com> - copyright 2019-2020.
# Source: Based on very helpful information from Anon, Simon, 
#         and others on the GnuCobol forum:
#         https://sourceforge.net/p/open-cobol/discussion/help/
# Date-written:  12/25/2019
# Last-revision: 01/10/2020
#
# Note: some parts of this script haven't been thoroughly tested yet.
# Please report any bugs and/or suggestions to cdgoldin@gmail.com.
# Thank you.
#
#########################################################################
# If you need/want to change the script, keep in mind that:
# All parameter variables are case-sensitive.
# Make sure you don't set any to NUL (i.e. set variable=), and make sure 
# you don't have any leading or trailing spaces (which will be treated as
# part of the variable), creating very difficult-to-debug problems.
#########################################################################

# Parameters that can be overridden on the command line
# -----------------------------------------------------
cobfolder="/home/$USER/cob";    #folder for unpacking tarballs
tarfolder="/home/$USER/cob";    #folder to receive tarballs
                    #Do NOT use "~/folder" - it won't work with this script!
libfolder="/usr/local";         #folder to install libraries
#This allows different library folders for different versions of Gnu Cobol
#Note - unlike the Windoze installation, the actual applications and their
#libraries do NOT get installed in "cobfolder", but rather in "libfolder" 
         
ISAM="BDB";                     #BDB, VBISAM or NODB
MATHLIB="MPIR";                 #MPIR or GMP

WIDE="no";                      #no=don't use WIDE, yes=use WIDE
                                #not supported with ncurses yet
                                    
cJSON="yes";                    #no=don't install cJSON, yes=install cSJON
     #I don't know how to disable cJSON from the GC3.1 build, so we always 
     #install it to avoid an error during the build.
     
XML2="yes";                     #no=don't install XML2, yes=install XML2

GC3="GC31";                     #GNU Cobol version to install
        
# Parameters that can only be changed by editing the script
# ---------------------------------------------------------
gc30version="gnucobol-3.0-rc1"
gc31version="gnucobol-3.1-dev"
gc32version="gnucobol-3.2-early-dev"

mpirversion="mpir-3.0.0";       #mpir version to install
gmpversion="gmp-6.1.2";         #GMP  version to install
BDBversion=db-6.2.38.NC         #BDB version to install
VBIversion=vbisam-2.0.1         #VBISAM version to install

#####################################################################
# global functions
#####################################################################
function pause ()
{ read -p "Press any key to continue" pause
  echo "";
}

function press_any_key ()
{ echo "";
  read -p "Press any key to continue - Control C to cancel " pause
  echo "";
}

function check_for_errors ()
{ echo "";
  read -p "Check for errors - Press any key to continue - Control C to cancel" pause
  echo "";
}

function yes_or_no () #messagetext
{ reply="no";
  while [[ "$reply" != "y" && "$reply" != "n" ]]
  do 
    read -p "$1? y/n " reply;
    if [ "$reply" == "Y" ]; then reply="y"; fi;
    if [ "$reply" == "N" ]; then reply="n"; fi;
  done
}

function get_parms () 
{ if [ ! "$parm" ]; then return; fi #no parameter

  rawparm="$parm"; parm=${parm^^} #upshift parameter
  
  if [ "$parm" == "BDB" ];    then ISAM="BDB";     return; fi
  if [ "$parm" == "VBISAM" ]; then ISAM="VBISAM";  return; fi
  if [ "$parm" == "NODB" ];   then ISAM="NODB";    return; fi
  
  if [ "$parm" == "MPIR" ];   then MATHLIB="MPIR"; return; fi
  if [ "$parm" == "GMP" ];    then MATHLIB="GMP";  return; fi
  
  if [ "$parm" == "WIDE" ];   then WIDE="yes";     return; fi
  
  if [ "$parm" == "CJSON" ];     then cJSON="yes"; return; fi
  if [ "$parm" == "CJSON=NO" ];  then cJSON="no";  return; fi
  
  if [ "$parm" == "XML2" ];      then XML2="yes";  return; fi
  if [ "$parm" == "XML2=NO" ];   then XML2="no";   return; fi
     
  if [ "$parm" == "GC30" ];   then GC3=GC30; return; fi
  if [ "$parm" == "GC31" ];   then GC3=GC31; return; fi
  if [ "$parm" == "GC32" ];   then GC3=GC32; return; fi
     
  if [ ${parm:0:2} == -C ]; then cobfolder=${rawparm:2}; return; fi
  if [ ${parm:0:2} == -T ]; then tarfolder=${rawparm:2}; return; fi
  if [ ${parm:0:2} == -L ]; then libfolder=${rawparm:2}; return; fi
    
  printf "\nInvalid parameter $rawparm\n"; 
  return 1
}

another="a";

function grab_a_beer ()
{ echo "This will take some time - grab $another beer";
  another="another";
} 

function mkdir_error () #directory name
{ 
  #if [ "$DISTRIB_ID" == "LinuxMint" ]; then
     echo "Trying 'sudo mkdir $1'"
     sudo mkdir "$1"
     if [ $? -eq 0 ]; then return 0; fi
  #fi   
  echo " "
  echo "Error creating $1 - Correct problem manually and restart"
  echo "If the directory is in root, you need to use 'sudo mkdir'"
  echo " "
  pause; exit 8
}  
#####################################################################
# Prelimaries
#####################################################################
for i in {1..9}
 do parm="$1"
    get_parms
    if [ $? -gt 0 ]; then pause; exit 8; fi 
    shift  
 done

if [ "$1" ]; then 
  printf "\nInvalid parameter $1\n"; 
  pause; exit 8
fi

if [ "$GC3" == "GC30" ]; then gc3version=$gc30version; fi
if [ "$GC3" == "GC31" ]; then gc3version=$gc31version; fi
if [ "$GC3" == "GC32" ]; then gc3version=$gc32version; fi

echo " "
echo "***************************************************************************"
printf "Install $gc3version to $cobfolder --> $libfolder\n"
echo " "
echo "with $ISAM $MATHLIB WIDE=$WIDE" cJSON=$cJSON XML2=$XML2
echo " "
echo "tarfolder=$tarfolder"
echo "cobfolder=$cobfolder"
echo "libfolder=$libfolder" 
echo "***************************************************************************"
echo "Are these parameters correct? If not, cancel, edit parameters, and re-run."

press_any_key

DISTRIB_ID=$(cat /etc/*-release 2>/dev/null | grep "DISTRIB_ID")
DISTRIB_ID="${DISTRIB_ID:11}" #get the DISTRIB_ID value from DISTRIB_ID=Xxxxxxx

if [ ! -d "$tarfolder" ]; then            
    mkdir "$tarfolder"
    if [ $? -gt 0 ]; then mkdir_error "$tarfolder"; fi
fi  

if [ ! -d "$cobfolder" ]; then
    mkdir "$cobfolder"
    if [ $? -gt 0 ]; then mkdir_error "$cobfolder"; fi
fi
 
# we store all the builds here, not just the final GnuCobol build!
# so DON'T delete it and mkdir anew!

if [ ! -d "$libfolder" ]; then
    mkdir "$libfolder"
    if [ $? -gt 0 ]; then mkdir_error "$libfolder"; fi
fi

if [ "$(ls -A $libfolder)" ]; then
   echo "If you are reinstalling to the same library folder, and changing from"
   echo "GMP to MPIR (or the inverse), you MUST remove the previous installation!"
   echo "If you are simply rebuilding the same installation, it isn't necessary."
   echo " "
   yes_or_no "Remove previous $libfolder installation"
else
   reply="n"
fi      

if [ "$reply" == "y" ]; then
   cd $cobfolder/$gmpversion
   sudo make uninstall
   sudo make distclean

   cd $cobfolder/$mpirversion
   sudo make uninstall
   sudo make distclean

   cd $cobfolder/$gc3version
   sudo make uninstall
   sudo make distclean
   
   echo " "
fi   

#####################################################################
# Install dependency packages required for GnuCOBOL 3.x
#####################################################################
yes_or_no "Install dependency packages required for GnuCOBOL 3.x"

if [ "$reply" == "y" ]; then
#  packages that are required (according to Simon)
   sudo apt install gcc libc-dev libdb-dev libgmp-dev libncurses-dev
   sudo apt install libtool-bin libxml2-dev make perl wget
 
#  If you want to tinker with GnuCOBOL you may add (according to Simon) 
   sudo apt install help2man icu-devtools
   sudo apt install libasan
   if [ $? -gt 0 ]; then sudo apt install libasan3; fi
   if [ $? -gt 0 ]; then sudo apt install libasan5; fi
   sudo apt install zlib1g-dev
   sudo apt install git subversion automake
   
#  Special cases
   if [ "$MATHLIB" == "MPIR" ]; then sudo apt install yasm; fi
   
#  if [[ "$DISTRIB_ID" == "Ubuntu"
#     || "$DISTRIB_ID" == "Xubuntu" ]]; then sudo apt install libasan5; fi
#  otherwise we installed libasan3, but now we install libasan and let the 
#  package manager decide which version to use. 

   if [ "$ISAM" == "VBISAM" ]; then sudo apt install unzip; fi 

   sudo apt-get install -f
   sudo apt-get autoremove #because Package Manager says to run it!
                           #althought it SHOULD be unnecessary with Mint 19
    
   press_any_key
fi 

#####################################################################
# Install GMP (GNU Multiple-Precision Arithmetic Library) 
#####################################################################
if [ "$MATHLIB" == "GMP" ]; then
   yes_or_no "Install GMP (GNU Multiple-Precision Arithmetic Library)";
else
   reply="n";
fi

if [ "$reply" == "y" ]; then 
   grab_a_beer
   cd $tarfolder
   wget -O $gmpversion.tar.xz https://gmplib.org/download/gmp/$gmpversion.tar.xz
   if [ $? -gt 0 ]; then
      rm $gmpversion.tar.xz #because wget created an empty file 
      printf "\nDownload of $gmpversion.tar.xz failed - correct problem and restart\n"
      pause; exit 8
   fi  
     
   cd "$cobfolder"
   tar xf "$tarfolder/$gmpversion.tar.xz"
   if [ $? -gt 0 ]; then
      printf "\nUnpacking of $gmpversion.tar.xz failed - correct problem and restart\n"
      pause; exit 8; fi
   
   cd "$gmpversion"
   ./configure --prefix=$libfolder --disable-static --enable-shared
   if [ $? -gt 0 ]; then check_for_errors; fi
   make
   if [ $? -gt 0 ]; then check_for_errors; fi
   make check
   if [ $? -gt 0 ]; then check_for_errors; fi
   sudo make install
   if [ $? -gt 0 ]; then check_for_errors; fi
   libtool --finish $libfolder/lib
   if [ $? -gt 0 ]; then check_for_errors; fi
   press_any_key
fi

##################################################################### 
# Install MPIR (Multiple Precision Integers and Rationals)
#####################################################################
if [ "$MATHLIB" == "MPIR" ]; then
   yes_or_no "Install MPIR (Multiple Precision Integers and Rationals)"
else
   reply="n";
fi

if [ "$reply" == "y" ]; then 
   grab_a_beer
   cd "$tarfolder"
   wget -O "$mpirversion.tar.bz2" http://mpir.org/$mpirversion.tar.bz2
   if [ $? -gt 0 ]; then
      rm $mpirversion.tar.xz #because wget created an empty file 
      printf "\nDownload of $mpirversion.tar.bz2 failed - correct problem and restart\n"
      pause; exit 8
   fi   
       
   cd "$cobfolder"
   tar xvjf "$tarfolder/$mpirversion.tar.bz2"
   if [ $? -gt 0 ]; then
      printf "\nUnpacking of $mpirversion.tar.xz failed - correct problem and restart\n"
      pause; exit 8; fi

   cd "$mpirversion"
   ./configure --prefix=$libfolder --disable-static --enable-shared --enable-gmpcompat
   if [ $? -gt 0 ]; then check_for_errors; fi
   make
   if [ $? -gt 0 ]; then check_for_errors; fi
   make check
   if [ $? -gt 0 ]; then check_for_errors; fi
   sudo make install
   if [ $? -gt 0 ]; then check_for_errors; fi
   press_any_key
fi

#####################################################################
# BDB is already installed - Install VBISAM database (VBI) 
#####################################################################
if [ "$ISAM" == "VBISAM" ]; then
   yes_or_no "Install VBISAM database (VBI)"
else
   reply="n";
fi

if [ "$reply" == "y" ]; then 
   printf "\nDownloading $VBIversion.zip\n\n"
   wget -O $tarfolder/$VBIversion.zip https://www.arnoldtrembley.com/$VBIversion.zip
   if [ $? -gt 0 ]; then 
       printf "\nDownload of $tarfolder/$VBIversion.zip failed - correct problem and restart\n"
       pause; exit 8
   fi

   printf "\nUnpacking tarball - this should take a few minutes\n\n"
   cd "$cobfolder"       
   unzip -q -o $tarfolder/$VBIversion.zip
   if [ $? -gt 0 ]; then
      printf "\nUnzip of $tarfolder/$VBIversion.zip failed - correct problem and restart\n"
      pause; exit 8; fi
  
   rm -i $tarfolder/$VBIversion.zip        

   printf "\nInstalling VBISAM\n"

   cd $cobfolder/$VBIversion
   chmod 777 ./configure
   ./configure --prefix=$libfolder
   if [ $? -gt 0 ]; then check_for_errors; fi
   make                          
   if [ $? -gt 0 ]; then check_for_errors; fi
   make check                    
   if [ $? -gt 0 ]; then check_for_errors; fi
   sudo make install             
   if [ $? -gt 0 ]; then check_for_errors; fi
   press_any_key
fi

#####################################################################
# Install the $gc3version compiler
#####################################################################
yes_or_no "Install the $gc3version compiler";

if [ "$reply" == "y" ]; then
   grab_a_beer

   if [ "$GC3" == "GC30" ];then url="https://sourceforge.net/projects/open-cobol/files/gnu-cobol/3.0/gnucobol-3.0-rc1.tar.gz/download"; fi
   if [ "$GC3" == "GC31" ];then url="https://ci.appveyor.com/api/projects/GitMensch/gnucobol-3-x/artifacts/gnucobol-3.1-dev.tar.gz?job=Image:%20Ubuntu1804"; fi

   if [ "$GC3" == "GC32" ];then url="https://ci.appveyor.com/api/projects/GitMensch/trunk/artifacts/gnucobol-3.2-early-dev.tar.gz?job=Image:%20Ubuntu1804"; fi

   cd "$tarfolder"   
   echo "Downloading $gc3version"
   wget -O $gc3version.tar.gz $url
   if [ $? -gt 0 ]; then 
      rm $gc3version.tar.xz #because wget creates an empty file 
      printf "\nDownload of $gc3version.tar.xz failed - correct problem and restart\n"
      pause; exit 8
   fi    

   cd "$cobfolder"
   tar xzf "$tarfolder/$gc3version.tar.gz"
   if [ $? -gt 0 ]; then
      printf "\nUnpacking of $gmpversion.tar.xz failed - correct problem and restart\n";
      pause; exit 8; fi 
   rm -i $tarfolder/$gc3version.tar.gz

   grep "IDENTIFICATION DIVISION." $cobfolder\$gc3version\tests\cobol85\newcob.val > /dev/null
   if [ $? -eq 1 ]; then rm        c:\minGW\msys\$gc3version\tests\cobol85\newcob.val; fi
   #if newcob.val is corrupt, remove it and let "make test" download and unpack it
   
   cJSON="yes" #always set cJSON to yes, to avoid config error!
   
   if [ "$cJSON" == "yes" ]; then
      echo "Downloading cJSON.c and cJSON.h"
      wget -O $cobfolder/$gc3version/libcob/cJSON.c https://raw.githubusercontent.com/DaveGamble/cJSON/master/cJSON.c 
      if [ $? -gt 0 ]; then
         rm $cobfolder/$gc3version/libcob/cJSON.c #because wget creates an empty file 
         printf "\nDownload of $cJson.c failed - correct problem and restart\n"
         pause; exit 8
      fi
      wget -O $cobfolder/$gc3version/libcob/cJSON.h https://raw.githubusercontent.com/DaveGamble/cJSON/master/cJSON.h 
      if [ $? -gt 0 ]; then
         rm $cobfolder/$gc3version/libcob/cJSON.h #because wget creates an empty file 
         printf "\nDownload of $cJson.h failed - correct problem and restart\n"
         pause; exit 8
      fi
   fi        

   cd "$gc3version";
   export LD_LIBRARY_PATH=$libfolder/lib:$LD_LIBRARY_PATH  #this was swapped, and not in ming!

                                    options="--without-db";
   if [ "$ISAM" == "BDB" ];    then options="--with-db"; fi
   if [ "$ISAM" == "VBISAM" ]; then options="--with-vbisam"; fi
   
   #if [ "$cJSON" == "no" ]; then options="$options --without-cJSON"; fi
   #unfortunately, there is no such option, but what is the correct one?
   
   if [ "$XML2" == "no" ]; then options="$options --without-xml2"; fi

   printf "\nconfiguring\n"
   ./configure --prefix=$libfolder $options "CPPFLAGS=-I$libfolder/include" "LDFLAGS=-L$libfolder/lib"
   if [ $? -gt 0 ]; then check_for_errors; fi 
   make
   if [ $? -gt 0 ]; then check_for_errors; fi
   make check
   if [ $? -gt 0 ]; then check_for_errors; fi
   make test
   if [ $? -gt 0 ]; then check_for_errors; fi
   sudo make install
   if [ $? -gt 0 ]; then check_for_errors; fi
   sudo ldconfig
   if [ $? -gt 0 ]; then check_for_errors; fi
fi

#####################################################################
# Run a test program (optional)
#####################################################################
cd "$cobfolder"

export COB_CONFIG_DIR=$cobfolder/$gc3version/config
export LD_LIBRARY_PATH=$libfolder/lib:$LD_LIBRARY_PATH
export LD_RUN_PATH=$libfolder/lib:$LD_RUN_PATH        
export LIBDIR=$libfolder/lib:$LIBDIR         
export PATH=$libfolder/bin:$PATH                   

echo " "
echo "PROGRAM-ID.HELLO.PROCEDURE DIVISION." > temp.cob
echo "DISPLAY 'Hello from GnuCOBOL'."      >> temp.cob
echo "DISPLAY ' '. DISPLAY 'The compiler works. Congratulations'." >> temp.cob
cobc -xj -free temp.cob
rm temp.cob

testfile="?"

until [ ! "$testfile" ]; do
   echo " "
   echo "Please enter the pathname of a Cobol source program to test"
   read -p "or just press Enter to skip test: " testfile

   if [ "$testfile" ]; then 
      echo " "
      if [ -f "$testfile.cob" ]; then
         testfile="$testfile.cob"
      else   
         if [ -f "$testfile.cbl" ]; then
            testfile="$testfile.cbl";
         #else
            # testfile is unaltered
         fi
      fi      
      grep -i "PROGRAM-ID." $testfile > /dev/null
      case $? in
         0) printf "\ncompiling and executing $testfile\n\n";
            cobc -x -j "$testfile";;
         1) printf "\n'$testfile' does not appear to be a valid Cobol program\n\n";;
         *) # grep will tell you there is no such file or directory
            # or other error message (if applicable)
            ;;
      esac
   fi
done    

#####################################################################
# Create set_env#.sh desktop file
#####################################################################
seqno=0; file="/home/$USER/Desktop/set_env"
while [ -f $file$seqno.sh ]; do seqno=$((seqno + 1)); done
file=$file$seqno.sh

function echo_to_file () # (ev, value)
{ echo "echo \$$1 | grep $2 > /dev/null"                         >> $file
  echo "if [ \$? -eq 1 ]; then export $1=$2:\$$1; fi"            >> $file
  echo " "                                                       >> $file 
}
      
echo "#!/bin/bash"                                                > $file
echo "# Set environment for $gc3version installed in $libfolder" >> $file
echo "#     MATHLIB=$MATHLIB ISAM handler=$ISAM"                 >> $file
echo "#--------------------------------------------------------" >> $file 
echo "# Usage: open a terminal window, and enter 'source $file'" >> $file
echo "# When you close the terminal window, it will 'unsource'"  >> $file
echo "# the four variables. If you want to be able to 'unsource'">> $file
echo "# without closing the terminal window, start a new (sub)shell" >> $file
echo "# first and source the script in the subshell. Then to reset"  >> $file
echo "# the environment to what it was before, just exit the subshell." >> $file
echo "# Feel free to rename the script to something more meaningful," >> $file
echo "# or move it to a different folder, if you wish"           >> $file
echo " "                                                         >> $file
       
echo "if [ \"\$0\" != \"bash\" ]; then"                          >> $file
echo "   echo \"This script must be run by entering 'source \$0'\""  >> $file
echo "   read -p \"press any key \""                             >> $file
echo "   exit 1"                                                 >> $file
echo "fi"                                                        >> $file

# As counter-intuitive as this seems, if the Desktop script is clicked on,
# $0 will be the script path. If you type the script name in a terminal,
# $0 will be the script name. If you type "bash scriptname" in a terminal,
# $0 will still be (just) the script name. But if you type "source scriptname",
# $0 will be "bash"!  
       
echo_to_file "LD_LIBRARY_PATH" "$libfolder/lib"
echo_to_file "LD_RUN_PATH"     "$libfolder/lib"     
echo_to_file "LIBDIR"          "$libfolder/lib"                  
echo_to_file "PATH"            "$libfolder/bin"   

echo " "                                                         >> $file
echo "printf \"\\nenvironment updated\\n\\n\""                   >> $file

chmod 777 $file
                   
echo " "
echo "Please 'source $file' before compiling or executing your"
echo "GnuCOBOL programs in a terminal window. If you wish, you can add" 
echo "it to ~/.bashrc so that it will be executed automatically when you"
echo "open a terminal window. You can also rename it to something more" 
echo "meaningful, or move it to a different folder, if you wish."
          
press_any_key

#####################################################################
# All good things come to an END
#####################################################################

printf "The end - La fin - Das Ende - Το τέλος - הסוף -  Fine - O fim - Конец\n\n"; 

######################### end of build_gc3 script ####################
