@echo off
@rem **************************************************************************
@rem * build_gc3_mingw.bat - Build GnuCOBOL 3.x for MinGW in Windoze 7 & 10
@rem * ------------------------------------------------------------------------
@rem * This script should work for current and future versions of GnuCobol, 
@rem * with only minor modification to the parameters and script.
@rem *
@rem * Syntax: build_gc3_mingw.bat [-cfolder_for_Cobol_installation]
@rem *                             [-tfolder_for_downloading_tarballs]
@rem *                             [-mfolder_in_which_mingw_is_installed]
@rem *                             [BDB | VBISAM | NODB] [GMP | MPIR]
@rem *                             [PDC3.4 | PDC3.9 | PDC4.1] [WINGUI] [WIDE]
@rem *                             [cJSON | "cJSON=no"] [XML2 | "XML2=no"]
@rem *                             [GC30 | GC31 | GC32]
@rem *
@rem * Defaults:
@rem *      cobfolder=c:\GNUCOBOL 
@rem *      tarfolder=%USERPROFILE%\Downloads 
@rem *      mingwfolder=c:\MinGW
@rem *       
@rem *      BDB GMP PDC3.9 cJSON XML2=no GC31
@rem *
@rem * All command-line parameters are optional. Folder names are passed to
@rem * the bash script, and therefore are case sensitive. The parameters
@rem * may be entered in any order, except that WINGUI must follow PDC4.1,
@rem * and WIDE must follow WINGUI to be valid.     
@rem *
@rem * Command-line parameters override the script variables defined below. In
@rem * general, command-line parameters are used to override variables that are
@rem * likely to change for each user, and for users who do multiple builds. 
@rem * Variables that are more static (such as program versions and download
@rem * urls) must be changed in the script.
@rem *
@rem * The batch file must be "Run as Administrator"!
@rem *
@rem * The script runs in a Windoze command prompt. It opens additional windows
@rem * to perform certain functions, including an MinGW/MSYS shell. The script
@rem * creates a bash file to pass on environmental variables, and to invoke 
@rem * build_gc3_mingw.sh for most other operations. The user must enter the
@rem * following command in the MSYS shell to execute the bash script when 
@rem * prompted: "build_gc3". The "build_gc3_mingw.sh" script must be located
@rem * in the same folder as this script (build_gc3_mingw.bat). 
@rem *
@rem * In general, you can cancel the script at any stage and restart it without
@rem * losing anything. Just reply "no" to running the steps that have already
@rem * completely successfully, and "yes" to the ones you need to rerun.
@rem * -----------------------------------------------------------------------
@rem * Author: Carl D. Goldin <cdgoldin@gmail.com> - copyright 2019-2020.
@rem * Source: Based on Arnold Trembly's GnuCobol Build guide
@rem *         (https://www.arnoldtrembley.com/GnuCOBOL.htm)
@rem           and very helpful information from Anon, Simon, David, 
@rem *         Ken, and others on the GnuCobol forum:
@rem *         https://sourceforge.net/p/open-cobol/discussion/help/
@rem * Date-written:  12/26/2019
@rem * Last-revision: 01/10/2020
@rem *
@rem * Note: some parts of this script haven't been thoroughly tested yet.
@rem * Please report any bugs and/or suggestions to cdgoldin@gmail.com.
@rem * Thank you.
@rem *
@rem **************************************************************************
@rem * If you need/want to change the script, keep in mind that:
@rem * All parameter variables are case-sensitive.
@rem * Make sure you don't set any to NUL (i.e. set variable=), and make sure 
@rem * you don't have any leading or trailing spaces (which will be treated as
@rem * part of the variable), creating very difficult-to-debug problems.
@rem **************************************************************************

@rem Parameters that can be overridden on the command line
@rem -----------------------------------------------------
set cobfolder=C:\GNUCOBOL
    rem folder for Cobol installation (this allows different versions)
    
set tarfolder=%USERPROFILE%\Downloads
    rem folder for downloading tarball and .exe files
    
set MinGWfolder=c:\MinGW
    rem folder for Building MinG

set ISAM=BDB
    rem BDB, VBISAM or NODB; you MUST specify one or the other

Set MATHLIB=GMP
    rem MPIR or GMP; you MUST specify one or the other

set pdcurses=PDCurses-3.9
    rem pdcurses version to install = PDCurses-3.4, PDCurses-3.9, PDCurses-4.1.0
    
set wingui=no
    rem no=install wincon, yes=install wingui (only valid with PDCurses-4.1.0)

set wide=no
    rem no=don't use PDC_WIDE, yes=use PDC_WIDE (only valid with PDCurses-4.1.0)
    
set cJSON=yes
    rem no=don't install cJSON, yes=install cSJON
    rem I don't know how to disable cJSON from the GC3.1 build, so we always 
    rem install it to avoid an error during the build.

set XML2=no
    rem no=don't install XML2, yes=install XML2
    rem I don't know how to provide XML2 for the GC3.1 build, so we always 
    rem install without it to avoid an error during the build.
    
set GC3=GC31
    rem GNU Cobol version to install

@rem Parameters that can only be changed by editing the script
@rem ---------------------------------------------------------

set gc30version=gnucobol-3.0-rc1
set gc31version=gnucobol-3.1-dev
set gc32version=gnucobol-3.2-early-dev
 
set mpirversion=mpir-3.0.0
    rem mpir version to install

set yasmversion=yasm-1.3.0-win32.exe
    rem yasm version to install (only used with MPIR)

set gmpversion=gmp-6.1.2
    rem gmp version to install

set BDBversion=db-6.2.38.NC
    rem BDB version to install

set VBIversion=vbisam-2.0.1
    rem BDB version to install

set MSYSfolder=%MinGWfolder%\MSYS
set userfolder=%MSYSfolder%\1.0\home\%USERNAME%
    rem do NOT change these two variables!
    
set bashscript=%~dp0build_gc3_mingw.sh
    rem drive and directory path of this script (including final \) + bash script

@rem *************************************************************************
@rem * Initialization routines
@rem *************************************************************************
openfiles > NUL 2>&1 
@IF %ERRORLEVEL% NEQ 0 (
    echo=
    ECHO This batch file MUST be run as administrator!
    pause
    exit /b 8)
  
 set /a count=1
:parm-loop
 set parm=%1
 call :GET-PARMS
 if %errorlevel% gtr 0 exit /b 8
 shift /1
 set /a count+=1
 if %count% leq 11 goto :parm-loop
 if %1. neq . (
   echo=
   echo Invalid parameter %1
   exit /b 8)
   
if %GC3%. equ GC30. set gc3version=%gc30version%
if %GC3%. equ GC31. set gc3version=%gc31version%
if %GC3%. equ GC32. set gc3version=%gc32version%
             
call :Trim-trailing-spaces %cobfolder% cobfolder %=because it is picking up a space somewhere=%
call :Trim-trailing-spaces %tarfolder% tarfolder %=in case it picks up a space somewhere too=%
call :Trim-trailing-spaces %MinGWfolder% MinGWfolder %=in case it picks up a space somewhere too=%
echo=
echo ***************************************************************************
echo Install %gc3version% to %cobfolder% folder
echo= 
echo with %ISAM% %MATHLIB% PDC%pdcurses:~3% WINGUI=%WINGUI% WIDE=%WIDE% cJSON=%cJSON% XML2=%XML2% 
echo=
echo tarfolder=%tarfolder% MinGWfolder=%Mingwfolder%  
echo *************************************************************************** 
echo Are these parameters correct? If not, cancel, edit parameters, and re-run.

call :PRESS-ANY-KEY
             
@rem **************************************************************************
@rem * Download and install wget for Windoze and MSYS
@rem **************************************************************************
:STEP1

if not exist c:\windows\system32\wget.exe (
   echo Please download wget.exe to your desktop from the webpage I'm opening now.
   echo Make sure to download the correct version [32 or 64 bit] for your system! 
   call :PRESS-ANY-KEY
   start /wait "Download wget" https://eternallybored.org/misc/wget/
   echo when you've done so...
   call :PRESS-ANY-KEY
   echo Moving wget.exe to c:\Windows\System32\ and %MinGWfolder%\msys\1.0\bin\wget.exe 
   move %userprofile%\desktop\wget.exe c:\Windows\System32\wget.exe
   move %userprofile%\desktop\wget.exe %MinGWfolder%\msys\1.0\bin\wget.exe
)

@rem **************************************************************************
@rem * Install MinGW and MSYS bash shell
@rem **************************************************************************
:STEP2
 choice /c YN /n /m  "Install MinGW and MSYS bash shell? Y/N? "
 if %errorlevel% equ 2 goto :STEP3

 echo=
 echo Download 32-bit version of MinGW - Minimalist GNU for Windows2.GNU
 echo Save the download in %tarfolder% and close the download window
 rem call :PRESS-ANY-KEY

 start /wait "DownloadMinGW " https://sourceforge.net/projects/mingw/files/Installer/mingw-get-setup.exe/download 
 echo when you've done so.. 
 rem call :PRESS-ANY-KEY

 if not exist %minGWfolder%\nul md %minGWfolder%

 echo=
 if %MinGWfolder%. neq C:\MinGW. (
 echo When you run mingw-get-setup.exe it will default to installing in C:\MinGW
 echo and you should override this with %MinGWfolder%!)
 echo=
 echo Use the GUI interface to select the mingw32-base and msys-base packages
 echo from the "Basic Setup" menu.
 echo=
 echo Then select the following components from the "All Packages" menu.
 echo (Some of them will already be selected with ming32-base and msys-base):
 echo=
 echo mingw32-bzip2.bin     mingw32-dos2unix.bin     mingw32-gettext.bin
 echo mingw32-gettext.dev   mingw32-gettext.dll      mingw32-libtool.bin
 echo mingw32-make.bin      msys-bash.bin            msys-m4.bin
 echo msys-make.bin         msys-perl.bin            msys-wget.bin
 echo=
 echo Then click "Installation > Apply changes" to apply all marked changes.
 echo The installation may take 5 to 15 minutes. Grab a beer.

 rem Open another window for the installation, so we can read the above
 rem instructions while configuring the setup! Question: is there anyway of
 rem automating this via command line or parameter file?
 
 start /wait "Mingw Setup " %tarfolder%\mingw-get-setup.exe 
 call :press-any-key

 if not exist %MinGWfolder%\bin\mingwm10.dll (
    echo %MinGWfolder%\bin\mingwm10.dll does not exist - cancelling.
    pause & exit /b 8)

 if not exist %MSYSfolder%\1.0\etc\fstab (
    echo %MSYSfolder%\1.0\etc\fstab does not exist - cancelling.
    pause & exit /b 8)

 echo Insure that fstab contains "%MinGWfolder% /mingw"
 Notepad %MSYSfolder%\1.0\etc\fstab 
 call :press-any-key
 
 echo If installation was successful, you can delete the setup file
 echo but you may want to keep it for future modifications.
 
 del %tarfolder%\mingw-get-setup.exe /p  

@rem **************************************************************************
@rem * Configure the Bash Shell
@rem **************************************************************************
:STEP3
 echo=
 choice /c YN /n /m  "Configure the bash shell? Y/N? "
 if %errorlevel% equ 2 goto :STEP4

 echo=
 echo Create an MSYS shortcut on your windows desktop as follows:
 echo   -Browse to %MSYSfolder%\1.0\msys.bat with Windows Explorer
         explorer %MSYSfolder%\1.0\
 echo   -Right click %MSYSfolder%\1.0\msys.bat
 echo   -Click "Send to > Desktop (create shortcut)"
 echo=
 echo Then right-click on the MSYS shortcut and click "Properties" to change:
 echo   General: Change shortcut name to MSYS 
 echo  Shortcut: Target:      "%MSYSfolder%\1.0\msys.bat -norxvt"
 echo                          -norxvt is essential!
 echo            Start in:    "%MSYSfolder%\1.0
 echo            Change icon: "%MSYSfolder%\1.0\msys.ico" (optional)
 echo            Advanced:    "Run as Administrator" and click "Apply"
 echo            This is necessary for Windoze Vista and Windows 7. It is also
 echo            necessary for W10 n order for the "sed" command to work.
 echo   Options: Enable Quick Edit Mode and Insert Mode
 echo      Font: Change the font to something that enables that window to fit
 echo            on your screen, e.g. "Lucida Console" with a font size of "10".
 echo    Layout: Screen Buffer Size: Width: 190  Height: (at least) 2000 lines
 echo            Window Size:        Width: 190  Height: 60 lines
 echo            This better allows us to view the voluminous output of "make".
 echo  Security: Edit ^> Administrators ^> Full Control
 echo=
 echo Double-click MYSYS icon to start the MSYS bash session.
 call :press-any-key
 
@rem **************************************************************************
@rem * Download and install all (?) other required tarball and .exe files
@rem * ------------------------------------------------------------------------
@rem * Automatically download every required tarball (based on the specified 
@rem * parameters) that is not in the %tarfolder% folder. To force a download, 
@rem * simply delete the tarball from the folder before running the script. 
@rem *
@rem * If we do "wget ...", there is no progress indicator!
@rem * If we do "start /wait wget...", there IS a progress indicator, but wget
@rem * closes the separate window before we can see any error message or stats.
@rem * wget --show-progress doesn't work
@rem * wget --progress=bar doesn't work
@rem * wget --progress=bar:force:noscroll doesn't work
@rem * wget --progress=dot works, so we'll use it, albeit bar would be better
@rem * wget --progress=dot:force:noscroll doesn't work
@rem * curl is installed in W10 and MSYS, but not W7, so it won't work here
@rem **************************************************************************
:STEP4

setlocal EnableDelayedExpansion
 
@rem Download GMP if needed
 
if %MATHLIB%. equ GMP. if not exist %tarfolder%\%gmpversion%.tar.xz (
   echo=
   echo Downloading %gmpversion%
   wget --progress=dot -O %tarfolder%\%gmpversion%.tar.xz https://gmplib.org/download/gmp/%gmpversion%.tar.xz
   if !errorlevel! gtr 0 (
       echo=
       echo Please download %gmpversion%.tar.xz to %tarfolder% manually
       exit /b 8)
)

@rem Download MPIR if needed
 
if %MATHLIB%. equ MPIR. if not exist %tarfolder%\%mpirversion%.tar.bz2 (
   echo=
   echo Downloading %mpirversion%
   wget --progress=dot -O %tarfolder%\%mpirversion%.tar.bz2 http://mpir.org/%mpirversion%.tar.bz2
   if !errorlevel! gtr 0 (
       echo=
       echo Please download %mpirversion%.tar.bz2 to %tarfolder% manually
       exit /b 8)   
   echo Downloading %yasmversion%
   wget --progress=dot -O %tarfolder%\%yasmversion% http://www.tortall.net/projects/yasm/releases/%yasmversion%
   if !errorlevel! gtr 0 (
       echo=
       echo Please download %yasmversion% to %tarfolder% manually
       exit /b 8)
)

@rem Download PDCurses if needed

if not exist %tarfolder%\%pdcurses%.tar.gz (
   echo=
   echo Downloading %pdcurses%
 
   if %pdcurses% equ PDCurses-3.4 ( 
      set url="https://sourceforge.net/projects/pdcurses/files/pdcurses/3.4/")
      
   if %pdcurses% equ PDCurses-3.9 (
      set url="https://fossies.org/linux/misc/PDCurses-3.9.tar.gz")
      
   if %pdcurses% equ PDCurses-4.1.0 ( 
      set url="https://github.com/Bill-Gray/PDCurses/archive/v4.1.0.tar.gz")
      
   wget --progress=dot -O %tarfolder%\%pdcurses%.tar.gz ^"!url!^"
   if !errorlevel! gtr 0 (
       echo=
       echo Please download %pdcurses%.tar.gz to %tarfolder% manually
       exit /b 8)
)
 
@rem Download BDB if needed

if %ISAM%. equ BDB. if not exist %tarfolder%\%BDBversion%.tar.gz (
   start /wait "Oracle Download " https://www.oracle.com/technetwork/database/database-technologies/berkeleydb/downloads/index-082944.html
   echo=
   echo [Create an Oracle user account if you don't already have one], log on, and download
   echo %BDBversion%.tar.gz to %tarfolder%\
   @rem We can't use wget, because it results in 'username/password authentication
   @rem failed with HTTP'. 
   call :PRESS-ANY-KEY
)

@rem Download VBISAM if needed

if %ISAM%. equ VBISAM. if not exist %tarfolder%\%VBIversion%.zip (
   echo=
   echo Downloading %VBIversion%.zip
   wget --progress=dot -O %tarfolder%\%VBIversion%.zip https://www.arnoldtrembley.com/%VBIversion%.zip
   if !errorlevel! gtr 0 (
       echo=
       echo Please download %vbiversion%.zip to %tarfolder% manually
       exit /b 8)
)  
@rem Download the GnuCOBOL compiler if needed

if exist %tarfolder%\%gc3version%.tar.gz (
   echo=
   choice /c YN /n /m  "Download the latest nightly build of %gc3version%? Y/N "
   if !errorlevel! equ 1 set reply=yes
   if !errorlevel! equ 2 set reply=no
) else                   set reply=yes

if %reply%. equ yes. (
   echo=
   echo Downloading %gc3version%
   
   if %GC3%. equ GC30. set url="https://sourceforge.net/projects/open-cobol/files/gnu-cobol/3.0/gnucobol-3.0-rc1.tar.gz/download"
   if %GC3%. equ GC31. set url="https://ci.appveyor.com/api/projects/GitMensch/gnucobol-3-x/artifacts/gnucobol-3.1-dev.tar.gz?job=Image:%%20Ubuntu1804"
   if %GC3%. equ GC32. set url="https://ci.appveyor.com/api/projects/GitMensch/trunk/artifacts/gnucobol-3.2-early-dev.tar.gz?job=Image:%%20Ubuntu1804"
      
   wget --progress=dot -O %tarfolder%/%gc3version%.tar.gz ^"!url!^"
   if !errorlevel! gtr 0 (
       echo=
       echo Please download %gc3version%.tar.gz to %tarfolder% manually
       exit /b 8)
)
setlocal DisableDelayedExpansion
endlocal
     
@rem newcob.val and cJSON are downloaded in the bash script, because they need
@rem to be downloaded (or moved) into folders that aren't built yet.

@rem **************************************************************************
@rem * Invoke the bash script to do the rest of the job
@rem **************************************************************************
:STEP5
 echo=
 choice /c YN /n /m  "Invoke the bash script to build GnuCOBOL with MSYS? Y/N "
 if %errorlevel% equ 2 goto :STEP6

 setlocal EnableDelayedExpansion
 call :CONVERT_FILENAME %bashscript%  & set bashscriptx=!lfn!
 call :CONVERT_FILENAME %MinGWfolder% & set MinGWfolderx=!lfn!
 call :CONVERT_FILENAME %tarfolder%   & set tarfolderx=!lfn!
 setlocal DisableDelayedExpansion
 endlocal

 @rem Note that folder and file names are in Windoze format in this script,
 @rem but are converted to Linux format when passed to MSYS

 echo #!/bin/bash                          > %userfolder%\build_gc3
 echo set -x #echo on                     >> %userfolder%\build_gc3
 echo export BDBversion="%BDBversion%"    >> %userfolder%\build_gc3
 echo export cJSON="%cJSON%"              >> %userfolder%\build_gc3
 echo export gc3version="%gc3version%"    >> %userfolder%\build_gc3
 echo export gmpversion="%gmpversion%"    >> %userfolder%\build_gc3
 echo export ISAM="%ISAM%"                >> %userfolder%\build_gc3
 echo export MATHLIB="%MATHLIB%"          >> %userfolder%\build_gc3
 echo export MinGWfolder="%MinGWfolderx%" >> %userfolder%\build_gc3
 echo export mpirversion="%mpirversion%"  >> %userfolder%\build_gc3
 echo export pdcurses="%pdcurses%"        >> %userfolder%\build_gc3
 echo export tarfolder="%tarfolderx%"     >> %userfolder%\build_gc3
 echo export VBIversion="%VBIversion%"    >> %userfolder%\build_gc3
 echo export wide="%wide%"                >> %userfolder%\build_gc3
 echo export wingui="%wingui%"            >> %userfolder%\build_gc3
 echo export XML2="%XML2%"                >> %userfolder%\build_gc3
 echo export yasmversion="%yasmversion%"  >> %userfolder%\build_gc3
 echo %bashscriptx%                       >> %userfolder%\build_gc3
 
 rem type %userfolder%\build_gc3 %=display the build_gc3 file for debugging=%
  
 start "MSYS" %USERPROFILE%\Desktop\msys.lnk
 echo **********************************************************************
 echo Please [double] click the MSYS icon on your desktop to start MSYS
 echo if it's not already started. Then type "build_gc3" in the MSYS window.
 echo **********************************************************************
 echo Wait for the MSYS build procedure to finish, then ...
 call :Press-any-key
    
@rem **************************************************************************
@rem * Prepare the %cobfolder% folder
@rem **************************************************************************
:STEP6
 echo=
 choice /c YN /n /m  "Prepare the %cobfolder% folder? Y/N? "
 if %errorlevel% equ 2 goto :STEP7

 if exist %cobfolder% rd /s /q %cobfolder% %=so we start with an empty folder=%
 md %cobfolder%

 xcopy %mingwfolder%\bin\*.*                   %cobfolder%\bin\ /s /e /q
                                           del %cobfolder%\bin\auto*.*
 xcopy %mingwfolder%\share\gnucobol\config\*.* %cobfolder%\config\ /s /e /q
 xcopy %mingwfolder%\share\gnucobol\copy\*.*   %cobfolder%\copy\ /s /e /q
 xcopy %mingwfolder%\lib\*.*                   %cobfolder%\lib\ /q 
 xcopy %mingwfolder%\lib\gcc\*.*               %cobfolder%\lib\gcc\ /s /e /q
 xcopy %mingwfolder%\libexec\gcc\*.*           %cobfolder%\libexec\gcc\ /s /e /q
 xcopy %mingwfolder%\include                   %cobfolder%\include\ /s /e /q
                                           del %cobfolder%\include\autosp*.*
 xcopy %MSYSfolder%\%gc3version%\extras\*.* %cobfolder%\extras\ /s /e /q
 
 echo=
 echo strip out unneeded GnuCOBOL components
 echo=
 cd /d %cobfolder%
 copy bin\strip* . 
 copy bin\libiconv* . 
 echo stripping - this might take a while
 strip -p --strip-debug --strip-unneeded bin\*.dll bin\*.exe lib\*.a
 del strip* libiconv*
 echo=

 echo Please download and add any files you wish (from the repository or
 echo elsewhere) to the %cobfolder% folder, now or any time in the future.
 echo=

@rem **************************************************************************
@rem * Create a "set_env.cmd" file (note that we create and use %COB_MAIN_DIR% 
@rem * instead of %cobfolder%, so that we can change the name of the folder
@rem * later without having to edit the set_env.cmd file!)
@rem * WARNING: IF YOU EDIT THIS PROCEDURE, NOTE THAT THERE CAN BE NO SPACES
@rem * BETWEEN THE VALUE IN AN "ECHO SET" COMMAND AND ">> set_env.cmd", even
@rem * if this makes things harder to read.
@rem **************************************************************************
echo=
echo creating set_env.cmd file

cd /d %cobfolder%

echo @echo off                                                               > set_env.cmd
echo echo=                                                                  >> set_env.cmd

set param=Setting environment for %gc3version% (%date%) with MinGW binaries
set param=%param:gnucobol-=GnuCOBOL %

echo echo %param%                                                           >> set_env.cmd

set param=(GCC 6.3.0, %pdcurses%
if %mathlib%. equ GMP.  set param=%param%, %gmpversion%
if %mathlib%. equ MPIR. set param=%param%, %mpirversion%
if %ISAM%. equ BDB.     set param=%param%, b%bdbversion%)
if %ISAM%. equ VBISAM.  set param=%param%, %vbiversion%)
if %ISAM%. equ NODB.    set param=%param%, NO ISAM)

rem cosmetic changes to match Simon's version (which looks better)
set param=%param:PDCurses-=PDCurses %
set param=%param:gmp-=GMP %
set param=%param:mpir-=MPIR %
set param=%param:bdb-=BDB %
set param=%param:vbisam-=VBISAM %

echo echo %param%                                                             >> set_env.cmd
echo=                                                                         >> set_env.cmd
echo :: Check if called already                                               >> set_env.cmd
echo :: if yes, check if called from here - exit, in any other case           >> set_env.cmd
echo :: raise warning and reset env vars                                      >> set_env.cmd
echo if not "%%COB_MAIN_DIR%%" == "" (                                        >> set_env.cmd
echo 	echo=                                                                 >> set_env.cmd
echo 	if "%%COB_MAIN_DIR%%" == "%%~dp0" (                                   >> set_env.cmd
echo 	   echo Information: batch was called already from "%%COB_MAIN_DIR%%" >> set_env.cmd
echo 	   echo              skipping environment setting...                  >> set_env.cmd  
echo 	   goto :cobcver                                                      >> set_env.cmd 
echo 	) else (                                                              >> set_env.cmd 
echo 	   echo Warning: batch was called before from "%%COB_MAIN_DIR%%"      >> set_env.cmd 
echo 	   echo          resetting COB_CFLAGS, COB_LDFLAGS                    >> set_env.cmd
echo 	   set COB_CFLAGS=                                                    >> set_env.cmd 
echo 	   set COB_LDFLAGS=                                                   >> set_env.cmd
echo 	)                                                                     >> set_env.cmd
echo )                                                                        >> set_env.cmd    
echo=                                                                         >> set_env.cmd
echo :: Get the main dir from the batch's position                            >> set_env.cmd
echo :: (only works in Windows NT environments or higher)                     >> set_env.cmd
echo set COB_MAIN_DIR=%%~dp0>> set_env.cmd
echo=                                                                         >> set_env.cmd
echo :: settings for cobc                                                     >> set_env.cmd
echo set COB_CONFIG_DIR=%%COB_MAIN_DIR%%config>>set_env.cmd 
echo set COB_COPY_DIR=%%COB_MAIN_DIR%%copy>>set_env.cmd
echo set COB_CFLAGS=-I"%%COB_MAIN_DIR%%include" %%COB_CFLAGS%%>>set_env.cmd
echo set COB_LDFLAGS=-L"%%COB_MAIN_DIR%%lib" %%COB_LDFLAGS%%>>set_env.cmd
echo set COB_CC=%%COB_MAIN_DIR%%bin\gcc.exe>>set_env.cmd
echo=                                                                         >> set_env.cmd
echo :: settings for libcob                                                   >> set_env.cmd
echo rem the following won't work in GnuCOBOL 3.0 if there are spaces in COB_MAIN_DIR >> set_env.cmd
echo set COB_LIBRARY_PATH=%%COB_MAIN_DIR%%extras>>set_env.cmd 
echo=                                                                        >> set_env.cmd   
echo :: Add the bin path of GnuCOBOL (including GCC) to PATH for further references >> set_env.cmd
echo path ^> temp                                                            >> set_env.cmd
echo findstr /i "%%COB_MAIN_DIR%%bin" temp > nul                             >> set_env.cmd
echo if %%errorlevel%% neq 0 set path=%%COB_MAIN_DIR%%bin;%%path%%>> set_env.cmd
echo=                                                                        >> set_env.cmd 
echo :: Compiler version output                                              >> set_env.cmd
echo :cobcver                                                                >> set_env.cmd
echo echo=                                                                   >> set_env.cmd                                    
echo cobc --i                                                                >> set_env.cmd
echo echo=                                                                   >> set_env.cmd
echo path                                                                    >> set_env.cmd
echo echo=                                                                   >> set_env.cmd

@rem **************************************************************************
@rem * Run a test program (optional)
@rem **************************************************************************
:STEP7
 cd /d %cobfolder%
 call set_env.cmd > nul
 set COB_CONFIG_DIR=%COB_MAIN_DIR%config
  
 echo=
 echo PROGRAM-ID.HELLO.PROCEDURE DIVISION. > temp.cob
 echo DISPLAY 'Hello from GnuCOBOL'.      >> temp.cob
 echo DISPLAY ' '. DISPLAY 'The compiler works. Congratulations'. >> temp.cob
 cobc -xj -free temp.cob
 del temp.cob
  
:STEP7-LOOP
 set testfile=
 echo=
 echo "Please enter the pathname of a Cobol source program to test"
 set /p testfile="or just press Enter to skip test: "
 
 if %testfile%. equ . goto :END
 
 echo=

 if exist %testfile%.cob (
    set testfile=%testfile%.cob
  ) else if exist %testfile%.cbl (
            set testfile=%testfile%.cbl)
 echo=
 
 findstr /I "PROGRAM-ID." %testfile% >nul 2>nul
 
 if %errorlevel% equ 0 (
    echo compiling and executing %testfile%
    echo=
    cobc -xj %testfile%
  ) else if exist %testfile% (
            echo %testfile% does not appear to be a valid Cobol program
            ) else echo %testfile% does not exist
 echo= 
 goto :STEP7-LOOP %=repeat until no program name entered=%

@rem **************************************************************************
@rem * All good things come to an END
@rem **************************************************************************
:END
 del temp*.*
 rem del %userfolder%\build_gc3 #no, leave it there, so we can test w/o running the entire script
 echo=
 echo The end - la fin - das Ende - to telos - hasof - fine - O fim - konets
 exit /b

@rem **************************************************************************
@rem * Internal subroutines
@rem **************************************************************************
:PRESS-ANY-KEY
 echo=
 echo Press any key to continue, Ctrl-C or Ctrl-Break to cancel
 pause > nul
 echo=
 exit /b

:GET-PARMS &rem parm is set above
 if %parm%. equ . exit /b %=no parameter=%
 
 set rawparm=%parm%
 
 call :UPSHIFT
     
 if %parm% equ BDB     (set ISAM=BDB
                        exit /b)
 if %parm% equ VBISAM  (set ISAM=VBISAM
                        exit /b)
 if %parm% equ NODB    (set ISAM=NODB
                        exit /b)

 if %parm% equ MPIR    (set MATHLIB=MPIR
                        exit /b)
 if %parm% equ GMP     (set MATHLIB=GMP
                        exit /b)

 if %parm% equ PDC3.4  (set pdcurses=PDCurses-3.4
                        exit /b)
 if %parm% equ PDC3.9  (set pdcurses=PDCurses-3.9
                        exit /b)
 if %parm% equ PDC4.1  (set pdcurses=PDCurses-4.1.0
                        exit /b)
                               
 if %parm% equ WINGUI if %pdcurses% equ PDCurses-4.1.0 (
                       set WINGUI=yes
                       exit /b)
                     
 if %parm% equ WIDE if %pdcurses% equ PDCurses-4.1.0 if %WINGUI% equ yes (
                       set WIDE=yes
                       exit /b)
 
 if %parm% equ CJSON      (set cJSON=yes
                           exit /b)     
                       
 if %parm% equ "CJSON=NO" (set cJSON=no
                           exit /b)                            
                       
 if %parm% equ XML2       (set XML2=yes
                           exit /b)    
                       
 if %parm% equ "XML2=NO"  (set XML2=no
                           exit /b)                                                                             

 if %parm% equ GC30  (set GC3=GC30
                      exit /b)
 if %parm% equ GC31  (set GC3=GC31
                      exit /b)     
 if %parm% equ GC32  (set GC3=GC32
                      exit /b)
                     
 if %parm:~0,2% equ -C (set cobfolder=%rawparm:~2% 
                        exit /b)
                        
 if %parm:~0,2% equ -T (set tarfolder=%rawparm:~2%
                        exit /b)
                        
 if %parm:~0,2% equ -M (set mingwfolder=%rawparm:~2%
                        exit /b)
 
 echo=
 echo Invalid parameter %rawparm%
 echo=
 exit /b 8 

:CONVERT_FILENAME &rem windoze_format_name --> linux_format_name (lfn)
 set wfn=%1%
 set lfn=%wfn:c:=/c%
 set lfn=%lfn:d:=/d%
 set lfn=%lfn:e:=/e%
 set lfn=%lfn:f:=/f%
 set lfn=%lfn:g:=/g%
 set lfn=%lfn:h:=/h%
 set lfn=%lfn:i:=/i%
 set lfn=%lfn:\=/%
 exit /b
 
:UPSHIFT &rem field_to_upshift (parm) is set above
SET parm=%parm:a=A%
SET parm=%parm:b=B%
SET parm=%parm:c=C%
SET parm=%parm:d=D%
SET parm=%parm:e=E%
SET parm=%parm:f=F%
SET parm=%parm:g=G%
SET parm=%parm:h=H%
SET parm=%parm:i=I%
SET parm=%parm:j=J%
SET parm=%parm:k=K%
SET parm=%parm:l=L%
SET parm=%parm:m=M%
SET parm=%parm:n=N%
SET parm=%parm:o=O%
SET parm=%parm:p=P%
SET parm=%parm:q=Q%
SET parm=%parm:r=R%
SET parm=%parm:s=S%
SET parm=%parm:t=T%
SET parm=%parm:u=U%
SET parm=%parm:v=V%
SET parm=%parm:w=W%
SET parm=%parm:x=X%
SET parm=%parm:y=Y%
SET parm=%parm:z=Y%
exit /b

:Trim-trailing-spaces &rem (%variable_to_be_trimmed% variable_to_be_trimmed) 
SET %2=%1
exit /b
